import java.awt.Color;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Label;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Vector;

public class PinBallGame extends Frame
{
   public static final int FrameWidth  = 400;
   public static final int FrameHeight = 400;

   private int   score;
   private Label scoreLabel;
   
   private Vector<PinBall>       balls;
   private Vector<PinBallTarget> targets;

   // ------------------------- PUBLIC INTERFACE -------------------------

   public PinBallGame()
   {
      setTitle( "Pinball Game Construction Kit" );
      setSize ( FrameWidth, FrameHeight );

      balls = new Vector<PinBall>();
      score = 0;

      scoreLabel = new Label( "score: 0" );
      add( "North", scoreLabel );

      addMouseListener( new MouseKeeper() );

      targets = new Vector<PinBallTarget>();
      buildWalls();
      buildPalette();
   }

   synchronized public void addScore( int value )
   { 
      score = score + value;
      scoreLabel.setText( "score = " + score );
   }

   public void paint( Graphics g )
   {
      paintLauncher( g );
      paintBalls   ( g );
      paintTargets ( g );
   }

   public Vector targets()
   {
       return targets;
   }

   // ------------------------- HELPER METHODS -------------------------

   protected void paintLauncher( Graphics g )
   {
      g.setColor ( Color.white );
      g.fillRect ( FrameWidth-40, FrameHeight-40, 30, 30 );
      g.setColor ( Color.red );
      g.fillOval ( FrameWidth-40, FrameHeight-40, 30, 30) ;
   }

   protected void paintBalls( Graphics g )
   {
      for (int i = 0; i < balls.size(); i++)
      {
         PinBall aBall = balls.elementAt( i );
         aBall.paint(g);
      }
   }

   protected void paintTargets( Graphics g )
   {
      for (int j = 0; j < targets.size(); j++)
      {
         PinBallTarget target = targets.elementAt( j );
         target.paint( g );
      } 
   }

   protected void buildWalls()
   {
      targets.addElement( new Wall(50, 50, 2, 350 ));
      targets.addElement( new Wall(50, 50, 340, 2 ));
      targets.addElement( new Wall(390, 50, 2, 380 ));
   }

   protected void buildPalette()
   {
      targets.addElement(new Hole( 30, 100 ));
      targets.addElement(new Peg( 30, 140, 100, this ));
      targets.addElement(new Peg( 30, 180, 200, this ));
      targets.addElement(new ScorePad( 30, 220, 100, this ));
      targets.addElement(new ScorePad( 30, 260, 200, this ));
      targets.addElement(new Spring( 15, 300 ));
      targets.addElement(new Wall( 30, 340, 2, 15 ));
   }

   // ------------------------- INNER CLASSES -------------------------

   private class MouseKeeper extends MouseAdapter
   {
      private PinBallTarget element;

      public void mousePressed( MouseEvent e )
      { 
         element = null;
         int x = e.getX();
         int y = e.getY();

         if ( (x > FrameWidth-40) && (y > FrameHeight -40) )
         {
            PinBall newBall = new PinBall( e.getX(), e.getY() );
            balls.addElement( newBall );
            Thread newThread = new PinBallThread( newBall, PinBallGame.this );
            newThread.start();
         } 
         else if ( x < 40 )
         {
            switch ( y / 40 )
            {
               case 2: element = new Hole( 0, 0 ); break;
               case 3: element = new Peg( 0, 0, 100, PinBallGame.this ); break;
               case 4: element = new Peg( 0, 0, 200, PinBallGame.this ); break;
               case 5: element = new ScorePad( 0, 0, 100, PinBallGame.this ); break;
               case 6: element = new ScorePad( 0, 0, 200, PinBallGame.this ); break;
               case 7: element = new Spring( 0, 0 ); break;
               case 8: element = new Wall( 0, 0, 2, 15 ); break;
            }
         }
      }

      public void mouseReleased( MouseEvent e )
      {
         int x = e.getX();
         int y = e.getY();

         if ( (element != null) && (x > 50) )
         {
            element.moveTo( x, y );
            targets.addElement( element );
            repaint();
         }
      }
   }
}
